module:    lander
synopsis:  A Dylan version of the classic lunar lander game
author:    Douglas M. Auclair, dauclair@hotmail.com
copyright: 2000, GPL
version:   1.0.1, Septeber 14, 2000

define function main(name, arguments)
  let playing? :: <boolean> = #t;

  assign-terminal-input-boundary-functions();
  
  while(playing?)
    format-out("You are aboard the Lunar Lander "
		 "about to leave the spacecraft.\n");
    detachment();
    init-lunar-lander();
    while(*height* > 0)
      print-values();
      compute-descent(consume-fuel());
    end while;
    print-result();
    playing? := play-again?();
  end while;
  format-out("Have a nice day.\n");
  exit-application(0);
end function main;

define function detachment() => ()
  format-out("Ready for detachment\n");
  force-output(*standard-output*);
  countdown();  // calls out to a C routine using melange
end function detachment;

define function init-lunar-lander() => ()
  *velocity* := 70;
  *fuel* := 500;
  *height* := 1000;
end function init-lunar-lander;

define function assign-terminal-input-boundary-functions() => ()
  *too-low* := method()
		   format-out("No cheating please! Fuel must be >= 0.\n");
		   #f;
	       end;
  *too-high* := method()
		    format-out("Sorry, you have not got that much fuel!\n");
		    format-out("%d\n", *fuel*);
		end;
end function assign-terminal-input-boundary-functions;

define function print-values() => ()
  let velocity-string = if(*height* > 0) "Landing v" else "V" end if;

  format-out("\nMeter readings\n-----------------------------------\n");
  format-table("Fuel (gal):",*fuel*);
  format-table(concatenate(velocity-string, "elocity (m/sec):"), *velocity*);
  format-table("Height (m):", *height*);
  format-out("\n");
end function print-values;

define function format-table(s :: <string>, datum :: <integer>) => ()
  let spaces = 35 - (size(s) + size(print-to-string(datum)));
  format-out("%s%s%d\n",s,make(<string>, size: spaces, fill: ' '),datum);
end function format-table;

define function consume-fuel() => (ans :: <integer>)
  let u :: <integer> = 0;
  if(*fuel* = 0)
    force-output(*standard-output*);
    wait-seconds(1);   // another C-melanged routine
  else
    u := read-number("How much fuel will you use? ", high: *fuel*);
  end if;
  u;
end function consume-fuel;

define function compute-descent(fuel-consumed :: <integer>) => ()
  let lunar-gravity = 2;
  let affected-velocity = (*velocity* + lunar-gravity) * 10;
  *height* := *height* - *velocity*;
  *velocity* := truncate/(affected-velocity - fuel-consumed * 2,10);
  *fuel* := *fuel* - fuel-consumed;
end function compute-descent;

define function print-result() => ()
  *height* := 0;
  print-values();
  format-out("%s\n", if(*velocity* > 5) 
		       "You have crashed."
		     else
		       "Congratulations!  This was a very good landing."
		     end if);
end function print-result;

define method play-again?() => (ans :: <boolean>)
  read-number("Would you like to play again? (0 = no, 1 = yes) ",
	out-of-range: method() 1 end) ~= 0;
end method play-again?;

// Invoke our main() function.
main(application-name(), application-arguments());

//-------------------------------------------------------
// Variable declarations
//-------------------------------------------------------

define variable *velocity* :: <integer> = 70;  // v
define variable *fuel* :: <integer> = 500;     // f
define variable *height* :: <integer> = 1000;  // h
